
import { createRoot } from 'react-dom/client'
import { ErrorBoundary } from 'react-error-boundary'
import App from './App.tsx'
import './index.css'

// Declare the APP_INITIALIZED property on the Window interface
declare global {
  interface Window {
    APP_INITIALIZED?: boolean;
    RENDER_ATTEMPTED?: boolean;
  }
}

// Error fallback component with more detailed error reporting
function ErrorFallback({error, resetErrorBoundary}) {
  console.error("Showing error fallback component:", error);
  
  return (
    <div className="flex flex-col items-center justify-center min-h-screen p-4 text-center bg-gray-100">
      <div className="max-w-md p-6 bg-white rounded-lg shadow-lg">
        <h2 className="mb-4 text-xl font-bold text-red-600">Etwas ist schiefgelaufen</h2>
        <p className="mb-4 text-gray-700">Wir entschuldigen uns für die Unannehmlichkeiten. Bitte versuchen Sie, die Seite neu zu laden.</p>
        <pre className="mb-4 p-2 bg-gray-100 text-xs overflow-auto rounded">
          {error && (typeof error === 'object' ? 
            (error.message || JSON.stringify(error, null, 2)) : 
            String(error))}
        </pre>
        <button
          onClick={resetErrorBoundary}
          className="px-4 py-2 text-white bg-blue-500 rounded hover:bg-blue-600"
        >
          Neu laden
        </button>
      </div>
    </div>
  );
}

// Initialize the app with more robust error capture
const initializeApp = () => {
  try {
    console.log('Initializing React application...');
    window.RENDER_ATTEMPTED = true;
    
    const root = document.getElementById("root");
    if (!root) {
      throw new Error('Root element not found, cannot mount React app');
    }
    
    const reactRoot = createRoot(root);
    
    reactRoot.render(
      <ErrorBoundary 
        FallbackComponent={ErrorFallback}
        onReset={() => {
          console.log('Error boundary reset triggered, reloading page');
          window.location.reload();
        }}
        onError={(error, info) => {
          console.error('React Error Boundary caught error:', error);
          console.error('Component stack:', info.componentStack);
          // Log additional details that might help with debugging
          console.error('Error occurred at:', new Date().toISOString());
          console.error('Browser:', navigator.userAgent);
          console.error('URL:', window.location.href);
        }}
      >
        <App />
      </ErrorBoundary>
    );
    
    // Mark that the app has initialized
    window.APP_INITIALIZED = true;
    console.log('React application initialized successfully');
  } catch (error) {
    console.error('Critical error during app initialization:', error);
    
    // Display a visible error message
    const root = document.getElementById("root");
    if (root) {
      root.innerHTML = `
        <div style="padding: 20px; font-family: sans-serif; text-align: center;">
          <h2 style="color: #d32f2f;">Application Error</h2>
          <p>We're sorry, but the application couldn't be loaded.</p>
          <p>Please try refreshing the page.</p>
          <button onclick="window.location.reload()" style="padding: 8px 16px; background: #2196f3; color: white; border: none; border-radius: 4px; cursor: pointer; margin-top: 16px;">
            Reload Page
          </button>
          <div style="margin-top: 20px; font-size: 12px; color: #666;">
            Error: ${error instanceof Error ? error.message : String(error)}
          </div>
        </div>
      `;
    }
    
    // Try to reload the page automatically after a delay
    setTimeout(() => {
      if (!window.APP_INITIALIZED) {
        console.warn('Attempting automatic reload after initialization failure');
        window.location.reload();
      }
    }, 5000);
  }
};

// Start the application with a slight delay to allow browser to settle
if (document.readyState === 'loading') {
  document.addEventListener('DOMContentLoaded', () => setTimeout(initializeApp, 100));
} else {
  setTimeout(initializeApp, 100);
}

// Fallback check to make sure rendering was attempted
setTimeout(() => {
  if (!window.RENDER_ATTEMPTED) {
    console.warn('No render attempt detected after timeout, forcing initialization');
    initializeApp();
  }
  
  // Additional check to ensure app has actually rendered content
  setTimeout(() => {
    const root = document.getElementById("root");
    if (!window.APP_INITIALIZED || (root && root.childNodes.length === 0)) {
      console.warn('App initialization check failed, forcing reload');
      window.location.reload();
    }
  }, 1000);
}, 2000);
